import fm from 'front-matter';
import MarkdownIt from 'markdown-it';
import { parseDocument, DomUtils } from 'htmlparser2';
import { Element } from 'domhandler';
import renderDom from 'dom-serializer';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var ExportedContent = /*#__PURE__*/function () {
  function ExportedContent() {
    this._exports = [];
    this._contextCode = '';
  }

  var _proto = ExportedContent.prototype;

  _proto.addContext = function addContext(contextCode) {
    this._contextCode += contextCode + "\n";
  };

  _proto.addExporting = function addExporting(exported) {
    this._exports.push(exported);
  };

  _proto["export"] = function _export() {
    return [this._contextCode, "export { " + this._exports.join(', ') + " }"].join('\n');
  };

  return ExportedContent;
}();
/**
let hList = [ {level:1}, {level:2}, {level:3}, {level:2}, {level:3}, {level:1}, {level:4}]
let nestedHeaders = createNestedHList(hList);
[
    {
        level: 1,
        children: [
            {
                level: 2,
                children: [{ level: 3 }]
            },
            {
                level: 2,
                children: [{ level: 3 }]
            }
        ]
    },
    {
        level: 1,
        children: [{ level: 4 }]
    }
]
 */


function createNestedHList(hList) {
  if (!hList.length) return []; // transform HItem to NestedHItem

  var nestedHeaders = hList.map(function (h) {
    return _extends({}, h, {
      children: []
    });
  }); // create virtual rootHeader as root Header

  var rootHeader = {
    level: 0,
    title: 'void',
    children: []
  };
  var prevHItem = rootHeader;
  var visited = [rootHeader]; // search visited from end to start, to find closest parent

  var findClosestParent = function findClosestParent(hItem) {
    for (var len = visited.length, i = len - 1; i >= 0; i--) {
      if (visited[i].level < hItem.level) {
        return visited[i];
      }
    } // add to elimilate ts error hint


    return visited[0];
  };

  nestedHeaders.forEach(function (curHItem) {
    if (prevHItem.level < curHItem.level) {
      prevHItem.children.push(curHItem);
      prevHItem = curHItem;
    } else if (prevHItem.level >= curHItem.level) {
      var closestParent = findClosestParent(curHItem);
      closestParent.children.push(curHItem);
      prevHItem = curHItem;
    }

    visited.push(curHItem);
  });
  return rootHeader.children;
}
function createMarkdown2HtmlMetadata(code, options) {
  if (options === void 0) {
    options = {
      html: true
    };
  }

  var fmContent = fm(code);
  var html = new MarkdownIt(options).render(fmContent.body);
  var rootDom = parseDocument(html);
  var hElements = rootDom.children.filter(function (rootSibling) {
    return rootSibling instanceof Element && ['h1', 'h2', 'h3', 'h4', 'h5', 'h6'].includes(rootSibling.tagName);
  });
  var hList = [];
  hElements.forEach(function (ele) {
    var hName = DomUtils.textContent(ele);
    ele.attribs['id'] = hName; // inject hash link to blog content

    var newEle = new Element('a', {
      href: '#' + hName
    }, undefined); // DomUtils.appendChild(newEle,new Text('#'))

    DomUtils.prependChild(ele, newEle);
    hList.push({
      level: parseInt(ele.tagName.replace('h', '')),
      title: hName
    });
  });
  return {
    attributes: fmContent.attributes,
    html: renderDom(rootDom),
    nestedHeaders: createNestedHList(hList)
  };
}

function markdown2html(code, options) {
  if (options === void 0) {
    options = {};
  }

  var metadata = createMarkdown2HtmlMetadata(code, options.markdownIt || {});
  var content = new ExportedContent();
  content.addContext("const attributes = " + JSON.stringify(metadata.attributes));
  content.addExporting('attributes'); // serialize injected hash html

  content.addContext("const html = " + JSON.stringify(metadata.html));
  content.addExporting('html');
  content.addContext("const nestedHeaders = " + JSON.stringify(metadata.nestedHeaders));
  content.addExporting('nestedHeaders');
  return content["export"]();
}

var plugin = function plugin(options) {
  if (options === void 0) {
    options = {};
  }

  return {
    name: 'vite-plugin-markdown2html',
    enforce: 'pre',
    transform: function transform(code, id) {
      if (!id.endsWith('.md')) return null;
      return {
        code: markdown2html(code, options)
      };
    }
  };
};

export default plugin;
export { createMarkdown2HtmlMetadata, createNestedHList, plugin };
//# sourceMappingURL=vite-plugin-md2html.esm.js.map
